//
//		OLED display IS evaluation board program
//
//			Initial.c
//			Initial setting program
//
//			Date	:	2008/08/28
//			Author	:	Y.Kinukawa
//			Ver		:	1.00
//
#include	"p24FJ32GA004.h"

#include	"GenericTypeDefs.h"
#include	"FSIO.h"

#include	"Io_port.h"
#include	"ExtFunc.h"

//
//		Allowing interruption
//
void	InterruptEnable( void )	{
	IEC0bits.T1IE		=	1;			//	T1 allowing interruption
	IEC0bits.T2IE		=	1;			//	T2 allowing interruption
	//	allowing interruption
	IEC1bits.INT1IE		=	1;
	IEC1bits.INT2IE		=	1;

	IEC1bits.CNIE		=	1;
}

//
//		Initial settings of Timer
//
void	TimerInit( void )	{

	//	Timer 1
	PR1				=	12500;			//	50ms
	T1CON			=	0x8020;			//	1000 0000 0011 0000

	IFS0bits.T1IF	=	0;				//	

	//	Timer 2
	PR2				=	1000;			//	4ms
	T2CON			=	0x8020;			//	1000 0000 0011 0000

	IFS0bits.T2IF	=	0;				//	

	//	Timer 3
	PR3				=	12500;			//	50ms (x 4)
	T3CON			=	0x8020;			//	1000 0000 0011 0000

}

//
//		Initial settings of A/D converter
//
void	AdcInit( void )	{

	CMCON			=	0;				//	comparator OFF

	AD1CON1			=	0x8044;			//	AD ON, integer output, T3 sampling, start of auto-conversion
	AD1CON2			=	0x040c;			//	AVdd,AVss, interruption at fourth conversion, MUXA only
	AD1CON3			=	0x1F05;			//	31 Tad auto-sample, Tad = 5*Tcy
 	AD1CHS			=	0x0000;			//	Channel selection (only lower bits)

	AD1PCFG			=	0xF9FF;			//	AN9 and 10 are analog pins.
	AD1CSSL			=	0x0600;			//	auto scan

	IEC0bits.AD1IE	=	0;				//	allowing interruption (void)
//	IEC0bits.AD1IE	=	1;				//	allowing interruption

}

//
//		initial settings of external interruption
//
void	ExtIntInit( void )	{
	//	pulse input
	TRISBbits.TRISB10	=	IN;
	TRISBbits.TRISB11	=	IN;

	//	settings of interruption ports
	RPINR0bits.INT1R	=	10;			//	right pulse of rotary encoder
	RPINR1bits.INT2R	=	11;			//	left pulse of rotary encoder

	//	interruption edge (rise-up)
	INTCON2bits.INT1EP	=	0;
	INTCON2bits.INT2EP	=	0;
}

//
//		initial settings of external SW inputs 
//
void	ExtSwInit( void )	{
	//	toggle SW
	TRISAbits.TRISA10	=	IN;			//	SW1
	TRISAbits.TRISA7	=	IN;			//	SW2

	//	IS-SW
	TRISAbits.TRISA0	=	IN;

	//	Dip SW
	TRISBbits.TRISB5	=	IN;
	TRISBbits.TRISB6	=	IN;
	TRISBbits.TRISB7	=	IN;
	TRISBbits.TRISB8	=	IN;

	//	SD_CD
	TRISCbits.TRISC9	=	IN;

	CNEN1	=	0x00c4;					//	CN2(IS-SW),CN6(),CN7()
	CNEN2	=	0x09c8;					//	CN19(SD_CD),CN22,23,24,27(DipSW)

	CNPU1	=	0x0004;					//	CN2(IS-SW)
	CNPU2	=	0x09c8;					//	CN19(SD_CD),CN22,23,24,27(DipSW)

}

//
//		LEDinitial settings of 
//
void	LedInit( void )	{
	//	LED1 control Port
	TRISAbits.TRISA4	=	OUT;
	//	LED2 control Port
	TRISBbits.TRISB4	=	OUT;
	//	LED3 control Port
	TRISAbits.TRISA8	=	OUT;
	//	LED4 control Port
	TRISAbits.TRISA3	=	OUT;
	//	LED5 control Port
	TRISAbits.TRISA2	=	OUT;
	//	LED6 control Port
	TRISCbits.TRISC2	=	OUT;
}

//
//		SPI1initial settings of 
//
void	SPI1Init( void )	{

	TRISCbits.TRISC8	=	IN;			//	SD_DI(RP24)
	TRISCbits.TRISC9	=	IN;			//	SD_CD(RP25)

	TRISBbits.TRISB9	=	OUT;		//	SD_CS(RP9)
	TRISCbits.TRISC6	=	OUT;		//	SD_DO(RP22)
	TRISCbits.TRISC7	=	OUT;		//	SDSCK(RP23)

	//	allocation of I/O ports
	RPINR20bits.SDI1R	=	24;			//	RP24  SPI1.SDI
	RPOR11bits.RP22R	=	7;			//	RP22  SPI1.SDO
	RPOR11bits.RP23R	=	8;			//	RP23  SPI1.SCK

	//	SPI mode settings
	SPI1CON1	=	0x027F;				//	0000 0010 0011 1110
//	SPI1CON1	=	0x027E;				//	0000 0010 0011 1110
//	SPI1CON1	=	0x027D;				//	0000 0010 0011 1100
	SPI1CON2	=	0x0000;				//	0000 0000 0000 0000

	PORTBbits.RB9		=	SET;
	IFS0bits.SPI1IF		=	0;
	SPI1STAT	=	0x8000;				//	1010 0000 0000 0000
	
	//	detection of SD card insertion
	SdCd	=	SD_CD;

	//	initialization of file system
	DevType	=	TGL_SW1;
	if( !SdCd || !FSInit() )	{
		DspFile	=	NON;
	}
	else	{
		DspFile	=	RESET;

		GetFileCount();

		FSchdir( (char *)DspDir[DevType][DspMode] );
	}
}

//
//		SPI2initial settings of 
//
void	SPI2Init( void )	{
	//	I/O settings
	TRISAbits.TRISA1	=	OUT;		//	RST Output
	TRISBbits.TRISB3	=	OUT;		//	SCK	Output
	TRISBbits.TRISB2	=	OUT;		//	SDO	OUtput
	TRISCbits.TRISC1	=	OUT;		//	SS	Output Port
	TRISCbits.TRISC0	=	OUT;		//	D/C	Output Port

	//	allocation of I/O port
	RPOR1bits.RP2R		=	10;			//	RP2  SPI2.SDO
	RPOR1bits.RP3R		=	11;			//	RP3  SPI2.SCK
	RPOR8bits.RP17R		=	12;			//	RP17  SPI2.SS

	//	SPI mode settings
	SPI2CON1	=	0x027F;				//	0000 0010 0111 1111
	SPI2CON2	=	0x0000;				//	0000 0000 0000 0000

	IFS2bits.SPI2IF		=	0;
	SPI2STAT	=	0x8000;				//	1000 0000 0000 0000

	//	OLED reset
	OLED_RST	=	HI;

	IsSw	=	IS_SW;
	PrgMode = MODE_A;
	DevType	=	TGL_SW1;
	if(DevType){
		buffer = BUFF;
	}
	else {
		buffer = BUFF_d;
	}	


}


