//
//		OLED display IS evaluation board program
//
//			Interrupt.c
//			Program for interruption processes
//
//			Date	:	2008/08/28
//			Author	:	Y.Kinukawa
//			Ver		:	1.00
//
#include	"p24FJ32GA004.h"

#include	"GenericTypeDefs.h"
#include	"FSIO.h"

#include	"Io_port.h"
#include	"ExtFunc.h"

//
//		Interruption by external switches
//
void  __attribute__((interrupt, no_auto_psv)) _CNInterrupt(void)	{
	unsigned	char	dip;
	//	clearing flag for interruption
	IFS1bits.CNIF	=	RESET;

	if( !MainLoop )
		return;
	PrgMode = MODE_A;
//	if( PrgMode )
//		return;
	
	//	reading rotary switch
	dip		=	(( EXT_DIP_B2 << 3 ) | ( EXT_DIP_B1 << 2 ) | ( EXT_DIP_B3 << 1 ) | EXT_DIP_B0 ) ^ 0x0f;
	if( dip == 0x09) {
			PrgMode = MODE_B;
			}		

	//	checking the change of status
	if( IS_SW != IsSw || SD_CD != SdCd || dip != DipSw )	{
		//	storing the status
		BfrIsSw	=	IsSw;			//	OLED-IS switch
		IsSw	=	IS_SW;

		BfrSdCd	=	SdCd;			//	SD card insertion
		SdCd	=	SD_CD;

		BfrDipSw	=	DipSw;		//	rotary switch
		DipSw		=	dip;

		//	OLED-IS switch press down
//	if( BfrIsSw != IsSw && IsSw ) {
//		Delayms(500);
	if( BfrIsSw != IsSw && IsSw ) {
		IncFileNum(); 
		PrgMode = MODE_B;
		DspFile = RESET;
	}
//	}

		//	rotary switch operation
		if( BfrDipSw != DipSw ){
		if( DipSw == 0x09) {
			PrgMode = MODE_B;
		} else if( BfrDipSw != DipSw )	{
			PrgMode = MODE_A;
			FileInterval	=	BASE_INTERVAL + INTERVAL_CNT * DipSw;
		}
		}
	}
}

//
//		INT1 interruption (right pulse of rotary encoder)
//
void  __attribute__((interrupt, no_auto_psv)) _INT1Interrupt(void)	{
	//	clearing flag for interruption
	IFS1bits.INT1IF	=	RESET;
	if( PrgMode )
		return;

	PrgMode	=	MODE_B;

	//	incremening file number
	if( !PrgMode )	{
		DecFileNum();

		DspFile	=	RESET;
	}
}

//
//		INT2 interruption (left pulse of rotary encoder)
//
void  __attribute__((interrupt, no_auto_psv)) _INT2Interrupt(void)	{
	//	clearing flag for interruption
	IFS1bits.INT2IF	=	RESET;
	if( PrgMode )
		return;

	PrgMode	=	MODE_B;

	//	decrementing file number
	if( !PrgMode )	{
		IncFileNum();

		DspFile	=	RESET;
	}
}

//
//		Timer 1 interruption i50msj
//
void  __attribute__((interrupt, no_auto_psv)) _T1Interrupt(void)	{
	//	clearing flag for interruption
	IFS0bits.T1IF	=	RESET;

	//	storing previous status
//	BfrPrgMode	=	PrgMode;
	BfrDspMode	=	DspMode;

	//	storing current status
	DevType	=	TGL_SW1;
	DspMode	=	TGL_SW2;

	if(DevType){
		buffer = BUFF;
	}
	else {
		buffer = BUFF_d;
	}	

	//	checking change in display mode
	if( BfrDspMode != DspMode )	{
		if( ModeChange )	{
			ModeChange	=	RESET;
		}
		else	{
			ModeChange	=	SET;
		}
	}
}

//
//		Timer 2 interruption i4msj
//
void  __attribute__((interrupt, no_auto_psv)) _T2Interrupt(void)	{
	//	clearing flag for interruption
	IFS0bits.T2IF	=	RESET;

	DevType	=	TGL_SW1;
	if(DevType){
		buffer = BUFF;
	}
	else {
		buffer = BUFF_d;
	}	


	//	MODE_A (automatic)
	if( PrgMode )	{
		IntervalCnt--;

		//	changes of pictures
		if( !IntervalCnt )	{
			if( Fd )	{
				FSfclose( Fd );
				//	clearing display parameters
				Fd	=	NULL;
				DspFile		=	NON;

				//	end of SD card access
				SdAccess	=	RESET;
				SetLED4( OFF );
			}

			IncFileNum();

			DspFile		=	RESET;
			DspOffset	=	RESET;
			IntervalCnt	=	FileInterval;

			OledAdrInit();
		}
	}

	//	LED4 (SD Access)
	if( SdAccess == SET )	{
		BlinkLED4();
	}

	//	LED5 (CPU RUN)
	PowLedCnt--;
	if( !PowLedCnt )	{
		PowLedCnt	=	BLINK250;
		BlinkLED5();
	}

}

//
//		A/D interruption
//
void  __attribute__((interrupt, no_auto_psv)) _ADC1Interrupt(void)	{

	//	clearing flag for interruption
	IFS0bits.AD1IF	=	RESET;

	//	reading the measured voltage
	Vcc		=	( ADC1BUF0 + ADC1BUF2 );
	Vref	=	( ADC1BUF1 + ADC1BUF3 );

	//	Vcc voltage check
	if( Vcc <= VCC_ERR_LEVEL )	{
		VccErr	=	BLINK100;
	}
	else	{
		VccErr	=	LEVEL_OK;
		SetLED1( ON );
	}

	//	Vref voltage check
	if( Vref <= VREF_ERR_LEVEL )	{
		VrefErr	=	BLINK100;
	}
	else	{
		VrefErr	=	LEVEL_OK;
		SetLED2( ON );
	}

}
