//
//		OLED display IS evaluation board program
//
//			Oled.c
//			main program
//
//			Date	:	2008/08/28
//			Author	:	Y.Kinukawa
//			Ver		:	1.00
//
#include	"p24FJ32GA004.h"

#include	"GenericTypeDefs.h"
#include	"FSIO.h"

#define		MAIN
#include	"Io_port.h"
#include	"ExtFunc.h"

//	#include	"ErrMsg.h"

/// configuration bits settings
_CONFIG1( JTAGEN_OFF & GCP_OFF & GWRP_OFF & BKBUG_OFF & COE_OFF & ICS_PGx1 & FWDTEN_OFF ) 
/// 8MHz*PLL4=32MHz (Full Speed)
	_CONFIG2( IESO_ON & FNOSC_FRCPLL & FCKSM_CSDCMD & OSCIOFNC_ON & IOL1WAY_OFF & I2C1SEL_PRI & POSCMOD_NONE )
//	_CONFIG2( IESO_ON & FNOSC_FRCDIV & FCKSM_CSDCMD & OSCIOFNC_ON & IOL1WAY_OFF & I2C1SEL_PRI & POSCMOD_HS )


//
//		initial settings
//
//
void	Initial( void )	{

	MainLoop	=	RESET;

	CLKDIV		=	0;

	ExtIntInit();				//	initial settings of external interruption
	ExtSwInit();				//	initial settings of external switch input
	LedInit();					//	initial settings of LED

	Fd			=	NULL;
	FileNum		=	0;
	PowLedCnt	=	BLINK250;

	SPI1Init();					//	SPI1 initial settings
	SPI2Init();					//	SPI2 initial settings

	OledInit();					//	OLED initial settings

	TimerInit();				//	Timer initial settings
	AdcInit();					//	A/D comparator initial settings

	SetLED();					//	LED initial status

	//	Ports settings
	DevType		=	TGL_SW1;
	DspMode		=	TGL_SW2;


	BfrIsSw		=	IsSw	=	IS_SW;
	BfrSdCd		=	SdCd	=	SD_CD;
	BfrDipSw	=	DipSw	=	(( EXT_DIP_B2 << 3 ) | ( EXT_DIP_B1 << 2 ) | ( EXT_DIP_B3 << 1 ) | EXT_DIP_B0 ) ^ 0x0f;

	IntervalCnt	=	FileInterval	=	BASE_INTERVAL + INTERVAL_CNT * DipSw;

	ModeChange	=	RESET;

	InterruptEnable();

	MainLoop	=	SET;

	if(DevType){
		buffer = BUFF;
	}
	else {
		buffer = BUFF_d;
	}	

}

//
//		Searching the number of bitmap files
//
//
void	GetFileCount( void )	{
	int			i;
	SearchRec	rec;

	//	Start of SD card access
	SetLED4( ON );
	SdAccess	=	SET;

	//	root directory setting
	FSchdir( ROOT_DIR );
	
	//	counting the number of files in each directory
	for( i = 0 ; i < 2 ; i++ )	{
		//	clearing table
		MaxFileCount[i]	=	0;

		//	directory set
		FSchdir( (char *)DspDir[DevType][i] );

		//	file search
		if( !FindFirst( FindFile[i], ATTR_ARCHIVE, &rec ) )	{
			MaxFileCount[i]++;		//	incrementing the number of files

			//	next file search
			while( !FindNext(&rec) )	{
				MaxFileCount[i]++;
			}
		}
		
		//	returning to root directory
		FSchdir( ROOT_DIR );
	}

	//	end of SD card access
	SetLED4( OFF );
	SdAccess	=	RESET;

}


//
//		incrementing the number of bitmap files
//
//
void	IncFileNum( void )	{
	//	conditions for incrementing the number of bitmap files
	if( SdCd && MainLoop )	{
		//	increment
		FileNum	=	( FileNum + 1 ) % MaxFileCount[DspMode];
	}
}

//
//		decrementing the nmuber of bitmap files
//
//
void	DecFileNum( void )	{
	//	conditions for decrementing the number of bitmap files
	if( SdCd && MainLoop )	{
		//	decrement
		FileNum	=	( FileNum - 1 + MaxFileCount[DspMode] ) % MaxFileCount[DspMode];
	}
}

//
//		setting of the names of bitmap files
//
//
void	SetFileName( void )	{
	int		i;
	int		num;

	num	=	FileNum + 1;

	//	file name setting	
	for( i = 0 ; *( BaseFile[DspMode] + i ) ; i++ )	{
		FileName[i]	=	*( BaseFile[DspMode] + i );
	}

	//	extension setting
	FileName[i++]	=	0x30 + ( num / 100 );
	FileName[i++]	=	0x30 + ( num / 10 ) % 10;
	FileName[i++]	=	0x30 + ( num % 10 );
	FileName[i]	=	NULL;
}

//
//		checking events
//
//
int		ChkEvent( void )	{
	int		ret		=	TRUE;

	//	monitoring SD card
	SetLED3( !SdCd );	

	//	file open
	if( !Fd && !DspFile && SdCd )	{
		//	file name setting
		SetFileName();

		OledAdrInit();

		//	display mode setting
		OledDspMode();

		FSchdir( (char *)ChgDir[DevType][DspMode] );

		//	opening bitmap files
		Fd	=	FSfopen( (const char *)FileName, "r" );

		if( !Fd )	{
			ret	=	FALSE;
		}
		else	{
			//	start of SD card access
			SetLED4( ON );

			//	display parameters settings
			DspFile		=	SET;
			DspLoop		=	0;
			SdAccess	=	SET;

			//	skipping header
			FSfread( DspBuff, 8, 1, Fd );
		}
	}

	//	SD card re-insert and re-detection	
	if( BfrSdCd != SdCd && SdCd )	{
		//	initialization of file system
		if( FSInit() )	{
			DspFile	=	RESET;
			BfrSdCd	=	SdCd;

			//	searching the number of files
			GetFileCount();

			//	changing directories for bitmap files
			FSchdir( (char *)DspDir[DevType][DspMode] );
		}
	}

	//	change of display modes
	if( !Fd && ModeChange )	{
		//	initializaion and reset of OLED
		OledInit();

		//	changing directories for bitmap files
		FSchdir( (char *)ChgDir[DevType][DspMode] );

		//	display parameters settings
		IntervalCnt	=	FileInterval;
		DspFile		=	RESET;
		FileNum		=	RESET;;
		
		ModeChange	=	RESET;
	}
	if(DevType){
		buffer = BUFF;
	}
	else {
		buffer = BUFF_d;
	}	

	return	ret;
}

//
//		reading bitmap data
//
//

int		GetDspData( void )	{
	int		ret	=	TRUE;

	if( SdCd )	{	//	reading bitmap file
		ret	=	FSfread( DspBuff, SIZE, buffer, Fd );
		DspOffset	=	0;

		//	read error	
		if( !ret && !FSfeof( Fd ) )	{
			ret	=	FALSE;
		}
	}
	
	return	ret;
}

//
//		output of bitmap data to OLED
//
//
void	SetDspData( void )	{
	if( SdCd )	{
		//	bitmap data transfer to OLED
		OledDataSend( &DspBuff[DspOffset], buffer );

		//	addition and adjustment of buffer offset
		DspOffset	=	( DspOffset + buffer ) % ( buffer * SIZE );

		//	end of buffer transfer
		if( !DspOffset )	{
			DspLoop++;	//	incrementing counter
	
			//	checking the completion of transfer
			if( DspLoop == LoopCnt[DspMode] )	{
				FSfclose( Fd );	//	closing file

				// clearing display parameters
				Fd	=	NULL;
				DspFile		=	NON;

				//	end of SD card access
				SdAccess	=	RESET;
				SetLED4( OFF );
			}
		}
	}
}

//
//		main roop
//
//
int		main( void )	{

	//	initialization
	Initial();

	//	main roop
	while( MainLoop )	{
		//	event check
		if( !ChkEvent() )	{
			continue;
		}

		//	reading display data
		if( !DspOffset && Fd )	{
			if( !GetDspData() )	{
				continue;
			}
		}
		//	transferring display data
		if( Fd )	{
			SetDspData();
		}
	}
	
	return	TRUE;
}
