//
//		OLED display IS evaluation board program
//
//			OledControll.c
//			OLED display control program
//
//			Date	:	2010/08/18
//			Author	:	K.Ohnuki(Y.Kinukawa)
//			Ver		:	2.00
//
#include	"p24FJ32GA004.h"

#include	"GenericTypeDefs.h"
#include	"FSIO.h"

#include	"Io_port.h"
#include	"ExtFunc.h"

//	OLED initial setting commands
const	unsigned char	SetupColumnAddress[]		=	{	0x03,	0x15,	0x10,	0x4f	};
const	unsigned char	SetupRowaddress[]			=	{	0x03,	0x75,	0x00,	0x2f	};
const	unsigned char	SetContrastColorA[] 		=	{	0x02,	0x81,	0x15	};
const	unsigned char	SetContrastColorB[] 		=	{	0x02,	0x82,	0x1a	};
const	unsigned char	SetContrastColorC[] 		=	{	0x02,	0x83,	0x17	};
const	unsigned char	MasterCurrentControl[]		=	{	0x02,	0x87,	0x0f	};
const	unsigned char	RemapColorDepth[]			=	{	0x02,	0xa0,	0x70	};
const	unsigned char	SetDisplayStartLine[]		=	{	0x02,	0xa1,	0x00	};
const	unsigned char	SetDisplayOffset[]			=	{	0x02,	0xa2,	0x10	};
const	unsigned char	SetDisplayMode[]			=	{	0x01,	0xa4	};
const	unsigned char	SetMultiplexRatio[]			=	{	0x02,	0xa8,	0x2f	};
const	unsigned char	DimModeSetting[]			=	{	0x05,	0xab,	0x12,	0x0c,	0x14,	0x12	};
const	unsigned char	SetDisplayDim[]				=	{	0x01,	0xac	};
const	unsigned char	SetDisplayOff[]				=	{	0x01,	0xae	};
const	unsigned char	SetDisplayNormal[]			=	{	0x01,	0xaf	};
const	unsigned char	SetMasterConfiguration[]	=	{	0x02,	0xad,	0x8e	};
const	unsigned char	PhasePeriodAdjustment[]		=	{	0x02,	0xb1,	0x44	};
const	unsigned char	DisplayClockDivider[]		=	{	0x02,	0xb3,	0xa0	};
const	unsigned char	EnableLinearGrayScale[]		=	{	0x01,	0xb9	};
const	unsigned char	SetPrechargeLevel[]			=	{	0x02,	0xbb,	0x12	};
const	unsigned char	SetVcomh[]					=	{	0x03,	0xbe,	0x3e	};

const	unsigned char	ColorDepth64k[]				=	{	0x02,	0xa0,	0x70	};
const	unsigned char	ColorDepth256[]				=	{	0x02,	0xa0,	0x30	};

//	OLED initial setting commands (for display type only)
const	unsigned char	SetupColumnAddress_d[]		=	{	0x03,	0x15,	0x16,	0x49	}; //display
const	unsigned char	SetupRowaddress_d[]			=	{	0x03,	0x75,	0x00,	0x23	}; //display
const	unsigned char	SetContrastColorA_d[] 		=	{	0x02,	0x81,	0x0e	}; //display
const	unsigned char	SetContrastColorB_d[] 		=	{	0x02,	0x82,	0x13	}; //display
const	unsigned char	SetContrastColorC_d[] 		=	{	0x02,	0x83,	0x12	}; //display
const	unsigned char	SetDisplayOffset_d[]			=	{	0x02,	0xa2,	0x1c	}; //display
const	unsigned char	SetMultiplexRatio_d[]			=	{	0x02,	0xa8,	0x23	}; //display
const	unsigned char	DimModeSetting_d[]			=	{	0x05,	0xab,	0x0b,	0x08,	0x0f,	0x12	}; //display
const	unsigned char	DisplayClockDivider_d[]		=	{	0x02,	0xb3,	0x30	}; //display
const	unsigned char	SetVcomh_d[]					=	{	0x03,	0xbe,	0x3c	}; //display	
//
//		OLED address settings
//
void	OledAdrInit( void )	{
	if(DevType){
	OledComSend( SetupColumnAddress );
	OledComSend( SetupRowaddress );
	}
	else {
	OledComSend( SetupColumnAddress_d );
	OledComSend( SetupRowaddress_d );
	}
}

//
//		OLED color depth settings (64K or 256 colors)
//
void	OledDspMode( void )	{

	if( DspMode )	{
		OledComSend( ColorDepth64k );
	}
	else	{
		OledComSend( ColorDepth256 );
	}
}

//
//		initial settings of OLED
//
void	OledInit( void )	{
	unsigned char	delay = 0x20;

	//	OLED reset
	OLED_RST	=	LOW;

	while( delay-- );

	OLED_RST	=	HI;

	//	initial settings
	OledAdrInit();

	if(DevType){
	OledComSend( SetContrastColorA );
	OledComSend( SetContrastColorB );
	OledComSend( SetContrastColorC );
	OledComSend( MasterCurrentControl );
	OledComSend( SetDisplayStartLine );
	OledComSend( SetDisplayOffset );
	OledComSend( SetDisplayMode );
	OledComSend( SetMultiplexRatio );
	OledComSend( DimModeSetting );
	OledComSend( SetMasterConfiguration );
	OledComSend( PhasePeriodAdjustment );
	OledComSend( DisplayClockDivider );
	OledComSend( EnableLinearGrayScale );
	OledComSend( SetPrechargeLevel );
	OledComSend( SetVcomh );
	}
	else {
	OledComSend( SetContrastColorA_d );
	OledComSend( SetContrastColorB_d );
	OledComSend( SetContrastColorC_d );
	OledComSend( MasterCurrentControl );
	OledComSend( SetDisplayStartLine );
	OledComSend( SetDisplayOffset_d );
	OledComSend( SetDisplayMode );
	OledComSend( SetMultiplexRatio_d );
	OledComSend( DimModeSetting_d );
	OledComSend( SetMasterConfiguration );
	OledComSend( PhasePeriodAdjustment );
	OledComSend( DisplayClockDivider_d );
	OledComSend( EnableLinearGrayScale );
	OledComSend( SetPrechargeLevel );
	OledComSend( SetVcomh_d );
	}

	//	display mode settings
	OledDspMode();

	OledComSend( SetDisplayNormal );

}

//
//		D/C line settings
//
//		Hi(1)	:	command transfer mode
//		Low(0)	:	data transfer mode
//
void	SetDCLine( unsigned char mode )	{

	//	D/C line settings (Hi/Low)
	PORTCbits.RC0	=	( mode & LSB );
}

//
//		reversing D/C line
//
void	ReverseDCLine( void )	{

	//	D/C line settings (Hi/Low)
	PORTCbits.RC0	^=	LSB;
}

//
//		SS settings
//		Hi(1)	:	slave not selected
//		Low(0)	:	slave selected
//

void	SetSSLine( unsigned char mode )	{

	//	SS line settings (Hi/Low)
	PORTCbits.RC1	=	( mode & LSB );
}

//
//		data transfer to OLED
//
void	OledWrite( unsigned char dat )	{

	//	waiting the completion of data transfer
	while( SPI2STATbits.SPITBF );

	//	selecting the slave
	SetSSLine( LOW );

	//	data transfer
	SPI2BUF	=	dat;

	//	releasing the slave
	SetSSLine( HI );
	
	ReverseDCLine();

}

//
//		commands transfer to OLED
//
//
void	OledComSend( const unsigned char *com )	{
	int		i;

	//	command transfer
	for( i = 0 ; i < *com ; i++ )	{
		//	command transfer mode
		SetDCLine( CMD );

		//	writing data
		OledWrite( *( com + i + 1 ) );
	}
}

//
//		data transfer to OLED
//
//
void	OledDataSend( char *dat, int len )	{
	int		i;

	//	commands transfer
	for( i = 0 ; i < len ; i++ )	{
		//	data transfer mode
		SetDCLine( DAT );

		OledWrite( *( dat + i ) );
	}
}
